package com.github.polpetta.mezzotre.orm;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;

import com.github.polpetta.mezzotre.helper.Loader;
import com.github.polpetta.mezzotre.helper.TestConfig;
import io.ebean.Database;
import io.ebean.typequery.PString;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.mockito.ArgumentCaptor;
import org.slf4j.LoggerFactory;
import org.testcontainers.containers.PostgreSQLContainer;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

@Tag("slow")
@Tag("database")
@Tag("velocity")
@Testcontainers
class CallbackQueryContextCleanerIntegrationTest {

  @Container
  private final PostgreSQLContainer<?> postgresServer =
      new PostgreSQLContainer<>(TestConfig.POSTGRES_DOCKER_IMAGE);

  private Database database;

  private BatchBeanCleanerService fakeBatchBeanCleanerService;
  private CallbackQueryContextCleaner callbackQueryContextCleaner;

  @BeforeEach
  void setUp() throws Exception {
    database =
        Loader.connectToDatabase(Loader.loadDefaultEbeanConfigWithPostgresSettings(postgresServer));

    fakeBatchBeanCleanerService = mock(BatchBeanCleanerService.class);

    callbackQueryContextCleaner =
        new CallbackQueryContextCleaner(
            fakeBatchBeanCleanerService,
            LoggerFactory.getLogger(CallbackQueryContextCleaner.class));
  }

  @Test
  @Timeout(value = 1, unit = TimeUnit.MINUTES)
  void shouldDeleteByGroupId() throws Exception {
    when(fakeBatchBeanCleanerService.removeAsync(eq("an id"), any()))
        .thenReturn(CompletableFuture.completedFuture(1));

    final Integer got = callbackQueryContextCleaner.removeGroupAsync("an id").get();

    final ArgumentCaptor<PString> pStringArgumentCaptor = ArgumentCaptor.forClass(PString.class);
    verify(fakeBatchBeanCleanerService, times(1))
        .removeAsync(eq("an id"), pStringArgumentCaptor.capture());
    assertEquals(1, got);
    assertEquals("entryGroup", pStringArgumentCaptor.getValue().toString());
  }

  @Test
  @Timeout(value = 1, unit = TimeUnit.MINUTES)
  void shouldDeleteById() throws Exception {
    when(fakeBatchBeanCleanerService.removeAsync(eq("an id"), any()))
        .thenReturn(CompletableFuture.completedFuture(1));

    final Integer got = callbackQueryContextCleaner.removeIdAsync("an id").get();

    final ArgumentCaptor<PString> pStringArgumentCaptor = ArgumentCaptor.forClass(PString.class);
    verify(fakeBatchBeanCleanerService, times(1))
        .removeAsync(eq("an id"), pStringArgumentCaptor.capture());
    assertEquals(1, got);
    assertEquals("id", pStringArgumentCaptor.getValue().toString());
  }
}
